@@ -8,6 +8,7 @@ import threading
 
 import pika
 import pika.exceptions
+import shortuuid
 from tornado import gen, concurrent, ioloop
 import topika
 
@@ -96,20 +97,25 @@ class RmqSubscriber(object):
         self._response_encode = encoder
 
         self._rpc_subscribers = {}
-        self._broadcast_subscribers = []
+        self._broadcast_subscribers = {}
         self._broadcast_queue = None  # type: topika.Queue
         self._broadcast_consumer_tag = None
 
         self._active = False
 
     @gen.coroutine
-    def add_rpc_subscriber(self, subscriber, identifier):
+    def add_rpc_subscriber(self, subscriber, identifier=None):
         # Create an RPC queue
         rpc_queue = yield self._channel.declare_queue(exclusive=True, arguments=self.RPC_QUEUE_ARGUMENTS)
-
-        yield rpc_queue.bind(self._exchange, routing_key='{}.{}'.format(defaults.RPC_TOPIC, identifier))
-        rpc_queue.consume(partial(self._on_rpc, subscriber))
-        self._rpc_subscribers[identifier] = rpc_queue
+        try:
+            identifier = yield rpc_queue.consume(partial(self._on_rpc, subscriber), consumer_tag=identifier)
+        except pika.exceptions.DuplicateConsumerTag:
+            raise kiwipy.DuplicateSubscriberIdentifier("RPC identifier '{}'".format(identifier))
+        else:
+            yield rpc_queue.bind(self._exchange, routing_key='{}.{}'.format(defaults.RPC_TOPIC, identifier))
+            # Save the queue so we can cancel and unbind later
+            self._rpc_subscribers[identifier] = rpc_queue
+            raise gen.Return(identifier)
 
     @gen.coroutine
     def remove_rpc_subscriber(self, identifier):
@@ -118,18 +124,29 @@ class RmqSubscriber(object):
         except KeyError:
             raise ValueError("Unknown subscriber '{}'".format(identifier))
         else:
+            yield rpc_queue.cancel(identifier)
             yield rpc_queue.unbind(self._exchange, routing_key='{}.{}'.format(defaults.RPC_TOPIC, identifier))
 
-    def add_broadcast_subscriber(self, subscriber):
-        self._broadcast_subscribers.append(subscriber)
+    @gen.coroutine
+    def add_broadcast_subscriber(self, subscriber, identifier=None):
+        identifier = identifier or shortuuid.uuid()
+        if identifier in self._broadcast_subscribers:
+            raise kiwipy.DuplicateSubscriberIdentifier("Broadcast identifier '{}'".format(identifier))
+
+        self._broadcast_subscribers[identifier] = subscriber
         if self._broadcast_consumer_tag is None:
             # Consume on the broadcast queue
-            self._broadcast_consumer_tag = self._broadcast_queue.consume(self._on_broadcast)
+            self._broadcast_consumer_tag = yield self._broadcast_queue.consume(self._on_broadcast)
+        raise gen.Return(identifier)
 
-    def remove_broadcast_subscriber(self, subscriber):
-        self._broadcast_subscribers.remove(subscriber)
+    @gen.coroutine
+    def remove_broadcast_subscriber(self, identifier):
+        try:
+            del self._broadcast_subscribers[identifier]
+        except KeyError:
+            raise ValueError("Broadcast subscriber '{}' unknown".format(identifier))
         if not self._broadcast_subscribers:
-            self._broadcast_queue.cancel(self._broadcast_consumer_tag)
+            yield self._broadcast_queue.cancel(self._broadcast_consumer_tag)
             self._broadcast_consumer_tag = None
 
     def channel(self):
@@ -198,7 +215,7 @@ class RmqSubscriber(object):
     def _on_broadcast(self, message):
         with message.process():
             msg = self._decode(message.body)
-            for receiver in self._broadcast_subscribers:
+            for receiver in self._broadcast_subscribers.values():
                 try:
                     receiver = utils.ensure_coroutine(receiver)
                     yield receiver(self, msg[messages.BroadcastMessage.BODY], msg[messages.BroadcastMessage.SENDER],
@@ -334,8 +351,9 @@ class RmqCommunicator(object):
         yield self._connection.close()
 
     @gen.coroutine
-    def add_rpc_subscriber(self, subscriber, identifier):
-        yield self._message_subscriber.add_rpc_subscriber(subscriber, identifier)
+    def add_rpc_subscriber(self, subscriber, identifier=None):
+        identifier = yield self._message_subscriber.add_rpc_subscriber(subscriber, identifier)
+        raise gen.Return(identifier)
 
     @gen.coroutine
     def remove_rpc_subscriber(self, identifier):
@@ -347,11 +365,14 @@ class RmqCommunicator(object):
     def remove_task_subscriber(self, subscriber):
         self._task_subscriber.remove_task_subscriber(subscriber)
 
-    def add_broadcast_subscriber(self, subscriber):
-        self._message_subscriber.add_broadcast_subscriber(subscriber)
+    @gen.coroutine
+    def add_broadcast_subscriber(self, subscriber, identifier=None):
+        identifier = yield self._message_subscriber.add_broadcast_subscriber(subscriber, identifier)
+        raise gen.Return(identifier)
 
-    def remove_broadcast_subscriber(self, subscriber):
-        self._message_subscriber.remove_broadcast_subscriber(subscriber)
+    @gen.coroutine
+    def remove_broadcast_subscriber(self, identifier):
+        yield self._message_subscriber.remove_broadcast_subscriber(identifier)
 
     @gen.coroutine
     def rpc_send(self, recipient_id, msg):
@@ -529,7 +550,7 @@ class RmqThreadCommunicator(kiwipy.Communicator):
         comm_thread.join()
         stop_future.result()
 
-    def add_rpc_subscriber(self, subscriber, identifier):
+    def add_rpc_subscriber(self, subscriber, identifier=None):
         coro = partial(self._communicator.add_rpc_subscriber, subscriber, identifier)
         return self._run_task(coro)
 
@@ -543,11 +564,13 @@ class RmqThreadCommunicator(kiwipy.Communicator):
     def remove_task_subscriber(self, subscriber):
         self._communicator.remove_task_subscriber(subscriber)
 
-    def add_broadcast_subscriber(self, subscriber):
-        self._communicator.add_broadcast_subscriber(subscriber)
+    def add_broadcast_subscriber(self, subscriber, identifier=None):
+        coro = partial(self._communicator.add_broadcast_subscriber, subscriber, identifier)
+        return self._run_task(coro)
 
-    def remove_broadcast_subscriber(self, subscriber):
-        self._communicator.remove_broadcast_subscriber(subscriber)
+    def remove_broadcast_subscriber(self, identifier):
+        coro = partial(self._communicator.remove_broadcast_subscriber, identifier)
+        return self._run_task(coro)
 
     def task_send(self, task, no_reply=False):
         self._ensure_running()
