@@ -1,5 +1,6 @@
 from __future__ import absolute_import
 from collections import deque
+import copy
 import logging
 import traceback
 import uuid
@@ -36,7 +37,7 @@ class BaseConnectionWithExchange(object):
     """
     DEFAULT_EXCHANGE_PARAMS = {'type': topika.ExchangeType.TOPIC}
 
-    def __init__(self, connection, exchange_name=defaults.MESSAGE_EXCHANGE, exchange_params=None):
+    def __init__(self, connection, exchange_name=defaults.MESSAGE_EXCHANGE, exchange_params=None, testing_mode=False):
         """
         :type connection: :class:`topika.Connection`
         :type exchange_name: str
@@ -45,7 +46,10 @@ class BaseConnectionWithExchange(object):
         super(BaseConnectionWithExchange, self).__init__()
 
         if exchange_params is None:
-            exchange_params = self.DEFAULT_EXCHANGE_PARAMS
+            exchange_params = copy.copy(self.DEFAULT_EXCHANGE_PARAMS)
+
+        if testing_mode:
+            exchange_params.setdefault('auto_delete', testing_mode)
 
         self._connection = connection
         self._exchange_name = exchange_name
@@ -83,13 +87,14 @@ class BaseConnectionWithExchange(object):
             if self.channel() is not None:
                 yield self.channel().close()
                 self._channel = None
-            self._connection = None
 
 
 class BasePublisherWithReplyQueue(object):
     """
-
+    A base class for any object that needs to be able to publish a message and to potentially expect a reply.
     """
+    # pylint: disable=too-many-instance-attributes
+
     DEFAULT_EXCHANGE_PARAMS = {'type': topika.ExchangeType.TOPIC}
 
     def __init__(self,
@@ -100,6 +105,7 @@ class BasePublisherWithReplyQueue(object):
                  decoder=defaults.DECODER,
                  confirm_deliveries=True,
                  testing_mode=False):
+        # pylint: disable=too-many-arguments
         """
         :param connection: The topika RMQ connection
         :type connection: :class:`topika.connection.Connection`
@@ -112,7 +118,10 @@ class BasePublisherWithReplyQueue(object):
         super(BasePublisherWithReplyQueue, self).__init__()
 
         if exchange_params is None:
-            exchange_params = self.DEFAULT_EXCHANGE_PARAMS
+            exchange_params = copy.copy(self.DEFAULT_EXCHANGE_PARAMS)
+
+        if testing_mode:
+            exchange_params.setdefault('auto_delete', testing_mode)
 
         self._exchange_name = exchange_name
         self._exchange_params = exchange_params
@@ -149,6 +158,7 @@ class BasePublisherWithReplyQueue(object):
         self._channel = yield self._connection.channel(
             publisher_confirms=self._confirm_deliveries, on_return_raises=True)
         self._channel.add_close_callback(self._on_channel_close)
+
         self._exchange = yield self._channel.declare_exchange(name=self.get_exchange_name(), **self._exchange_params)
 
         # Declare the reply queue
@@ -185,6 +195,15 @@ class BasePublisherWithReplyQueue(object):
 
     @gen.coroutine
     def publish(self, message, routing_key, mandatory=True, immediate=False):
+        """
+        Send a fire-and-forget message i.e. no response expected.
+
+        :param message: The message to send
+        :param routing_key: The routing key
+        :param mandatory: If the message cannot be routed this will raise an UnroutableException
+        :param immediate: Return the message if it cannot be queued immediately
+        :return:
+        """
         result = yield self._exchange.publish(
             message, routing_key=routing_key, mandatory=mandatory, immediate=immediate)
         raise gen.Return(result)
